#ifndef AVEUGLE_H_
#define AVEUGLE_H_

#include <eoInit.h>
#include "../Individu/individu.h"
#include "../Individu/Genotype.h"
/*! @class eoCPTEO_V1Init : public eoInit<GenotypeT>
 * @author Jacques Bibai
 * @brief Definition de la maniere de construire un individus (liste de gares) 
 */
template <class GenotypeT>
class eoCPTEO_V1Init: public eoInit<GenotypeT> {
	private:  
   
	public:
		/**
		 * constructeur par defaut vide
		 */
	  eoCPTEO_V1Init(){	    
		
	  }
	  
	  
 	
 	/**
 	 * @brief operateur permettant l'initialisation d'un individu. Ici la taille de l'individu (nombre de stations) est tiree aleatoirement par rapport a un parametre.
 	 * Ensuite nous construisons les gares intermediaire en tirant de facon aleatoire la taille, le predicat a utilise et les termes    
 	 * @param _genotype : reference vers un individu
 	 */ 
	  void operator()(GenotypeT & _genotype)
	  {
	  		// initialisation du vecteur de choix des predicats
	  		CPTPredicateArray preds = cptGetPredicates();
	  		std::vector<unsigned>jeu;
	  		jeu.resize(global.getPredicateSize());
	  		//initialisation avec la valeur exacte de la table des predicats
	  		for(size_t i=0;i<jeu.size();i++) 
	  		{	size_t j;
	  			for(j=0;j<(unsigned)preds.nb;j++)
	  				if(strcmp(cptGetPredicateName(preds.elems[j]),global.getPredicate(i).c_str())==0)break;
	  			jeu[i]=j;
	  			
	  		}
	  		// choix aleatoire et initialisation du nombre de stations
	    	unsigned nbStations = 1 + eo::rng.random(global.problemGoal.size()/global.getDistance());		
			_genotype.getGenome().resize(nbStations);
			/*******************************************************************************************
			 *                                   debut de construction d'un individu                   *
			 *******************************************************************************************/
			 unsigned tailleGare;
			 unsigned posPredicate;
			bool test;
			 std::vector<string> deja;
			for(size_t i=0; i<_genotype.getGenome().size();i++)
			{
				/*******************************************************
				 *         debut de construction d'une gare            *
				 *******************************************************/
				// choix de la taille et mise a jour de la gare
				tailleGare=1+ eo::rng.random(global.getDistance()-1);
				Gene gene;
				gene.getGene().resize(tailleGare);
				
				for(size_t j=0;j<gene.size();j++)
				{
					do
					{
						test=false;
						//choix du predicat 
						posPredicate=rng.random(jeu.size());
						//debut tirage aleatoire de termes 
						//	cout << posPredicate << "***************\n";				
						std::vector<CPTTermArray> terms;
				    	terms.resize(cptGetArity(preds.elems[jeu[posPredicate]]));
				    	//initialisation de la table des termes
				    	for(size_t k=0; k<terms.size();k++){
				    		terms[k]=cptGetPossibleTerms(preds.elems[jeu[posPredicate]],k);
				    	}
				    	// choix des termes
				    	std::vector<CPTTerm> argument;
				    	argument.resize(terms.size());
				    	for(size_t k=0;k<argument.size();k++)
				    	{
				    		argument[k]=terms[k].elems[rng.random(terms[k].nb)];
				    	}
				    	
				    	// construction du fluent correspondant 
				    	std::string fluentChar("(");
				    	fluentChar+=cptGetPredicateName(preds.elems[jeu[posPredicate]]);
				    	for(size_t k=0;k<argument.size();k++)
				    	{
				    		fluentChar+=" ";
				    		fluentChar+=cptGetTermName(argument[k]);
				    		
				    	}
				    	fluentChar+=")";
				    	
				    	CPTFluent fluent=cptGetFluentFromName((char *)fluentChar.c_str());
				    	if(fluent!=NULL)
				    	{	int test_deja=0;
				    		if(gene.mutex2(cptGetIndexFromFluent(fluent),j)==0 && gene.notExist(cptGetIndexFromFluent(fluent))) {
				    			CPTTermArray terms = cptGetTerms(fluent);
				    		 	for(size_t t=0;t<deja.size();t++){
				    		 		if(terms.nb>0)if(strcmp(deja[t].c_str(),cptGetTermName(terms.elems[0]))==0)test_deja=1;
				    		 	}
				    		 	 if(test_deja==0){
				    		 		 	gene.set(j,cptGetIndexFromFluent(fluent));
				    		 		 if(terms.nb>0)	deja.push_back(cptGetTermName(argument[0]));
				    		 	}
				    			
				    		}
				    		else test=true;
				    	}
				    	else test=true;
					}while(test);
					
				}
				/**************************************************************
				 *      fin de construction de la gare : ajout de la gare     *
				 **************************************************************/ 
				 
				_genotype.set(i,gene);
				
			}
			
	    	
	    	
						 
			 
	   		 _genotype.invalidate();	   
	  }



};
#endif /*AVEUGLE_H_*/
